# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 James E. King III (@jeking3) <jking@apache.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import warnings

from contextlib import AbstractContextManager
from typing import Dict


class EnvironmentVariableChangedWarning(ResourceWarning):
    """
    An environment variable being managed by a TemporaryEnvironment was
    changed by the context it was managing.
    """

    pass


class TemporaryEnvironment(AbstractContextManager):
    """
    Manages a temporary environment.

    Sets or removes environment variables when entering, and will undo those
    changes on exit.  It does not save the entire environment, it will only
    modify the items it was directed to modify.

    If an environment variable is part of the directives and is changed
    by the context being managed, a warning is issued.
    """

    def __init__(self, directives: Dict[str, str], restore_if_changed: bool = True):
        """
        Initializer.

        Arguments:
            directives (Dict[str, str]):
                A list of environment variables to add/overwrite or
                remove.  To remove an environment variable set the value
                to None.  To remove the content of an environment variable
                but leave it defined, set the value to str().
            restore_if_changed (bool):
                Restore each tracked environment variable to the original state
                even if it was changed during the context.
        """
        # arguments
        self.directives = directives
        self.restore_if_changed = restore_if_changed

        # internals
        self._originals = {}  # key: envvar name, value: string or None (not there)

    def __enter__(self):
        """
        Entering scope.

        Add any additions and remove any removals from the environment.
        Save the original values for restoration later.
        """
        for ename, dvalue in self.directives.items():
            self._originals[ename] = os.environ.get(ename)
            if dvalue is not None:
                os.environ[ename] = dvalue
            elif ename in os.environ:
                del os.environ[ename]

    def __exit__(self, *exc):
        """
        Leaving scope.

        Check for any changes and issue warnings if required, then restore
        things to their proper state if required.
        """
        for ename, ovalue in self._originals.items():
            dvalue = self.directives[ename]
            evalue = os.environ.get(ename)
            if evalue != dvalue:
                warnings.warn(ename, EnvironmentVariableChangedWarning)
                if not self.restore_if_changed:
                    continue
            if evalue != ovalue:
                if ovalue is not None:
                    os.environ[ename] = ovalue
                else:
                    del os.environ[ename]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 James E. King III (@jeking3) <jking@apache.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from pathlib import Path

from pkg_resources import parse_version
from setuptools import setup

# Configurables

name = "tempenv"
description = "Environment Variable Context Manager"
packages = ["tempenv"]
versionfile = "tempenv/version.py"

# Everything below should be cookie-cutter


def get_requirements(name: str) -> list:
    """
    Return the contents of a requirements file

    Arguments:
      - name: the name of a file in the requirements directory

    Returns:
      - a list of requirements
    """
    return read_file(Path(f"requirements/{name}")).splitlines()


def read_file(path: Path) -> str:
    """
    Return the contents of a file

    Arguments:
      - path: path to the file

    Returns:
      - contents of the file
    """
    with path.open() as desc_file:
        return desc_file.read().rstrip()


def version():
    """
    Return the version string for the package stored in versionfile.
    """
    _version = {}
    exec(read_file(Path(versionfile)), _version)
    return str(parse_version(_version["__version__"]))


requirements = {}
for type in ["run", "setup", "test"]:
    requirements[type] = get_requirements(type)

setup(
    name=name,
    version=version(),
    python_requires=">=3.6",
    description=description,
    long_description=read_file(Path("README.rst")),
    long_description_content_type="text/x-rst",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Security",
        "Topic :: System :: Shells",
        "Topic :: Utilities",
    ],
    keywords="temporary, environment variable, context manager",
    download_url="https://github.com/jeking3/tempenv/archive/master.zip",
    url="https://github.com/jeking3/tempenv",
    author="James E. King III",
    author_email="jking@apache.org",
    license="Apache License 2.0",
    install_requires=requirements["run"],
    setup_requires=requirements["setup"],
    tests_require=requirements["test"],
    test_suite="unittest",
    packages=packages,
    include_package_data=True,
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 James E. King III (@jeking3) <jking@apache.org>
#

__all__ = ["__version__", "EnvironmentVariableChangedWarning", "TemporaryEnvironment"]

from .version import __version__
from .tempenv import EnvironmentVariableChangedWarning
from .tempenv import TemporaryEnvironment



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 James E. King III (@jeking3) <jking@apache.org>
#
__author__ = "James E. King III"
__copyright__ = "Copyright (C) 2019 James E. King III (@jeking3) <jking@apache.org>"
__license__ = "Apache Software License, Version 2.0"
__version__ = "0.1.0"
